// logconv.cpp
//
// Copyright (C) 2000 Robert Barron, KA5WSS
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#if defined(LC_GUI) && defined(LC_NT)
#include <windows.h>
#endif
#include <stdio.h>
#if !defined(LC_GUI) || !defined(LC_NT)
#include <string.h>
#endif

#define STRING_BUFFER_SIZE    2048

#include "logconv.h"

char *progName = "LogConv";

RLogConv::RLogConv()
{
	memset(szInputFileName, '\0', STRING_BUFFER_SIZE);
	memset(szOutputFileName, '\0', STRING_BUFFER_SIZE);
	memset(&fileOptions, '\0', sizeof(RQsoFile::QSOFILEOPTIONS));

	nQSOsConverted = 0;
	FromFile = NULL;
	ToFile = NULL;

	return;
}

RLogConv::~RLogConv()
{
	if (FromFile)
	{
		FromFile->close();
		delete FromFile;
	}

	if (ToFile)
	{
		ToFile->close();
		delete ToFile;
	}

	return;
}

int RLogConv::setInputFile(char *szInputFile, int *nFileType)
{
	strcpy(szInputFileName, szInputFile);

	switch (getInputLogType())
	{
		case LOG_TYPE_CT7:
			FromFile = new RCtFile();
			break;
		case LOG_TYPE_CT8:
			FromFile = new RCt8File();
			break;
		case LOG_TYPE_CT9:
			FromFile = new RCt9File();
			break;
		case LOG_TYPE_WRTC:
			FromFile = new RWrtcFile();
			break;
		case LOG_TYPE_NALOG:
			FromFile = new RNaFile();
			break;
		case LOG_TYPE_NA9LOG:
			FromFile = new RNa9File();
			break;
		case LOG_TYPE_TRLOG:
			FromFile = new RTrFile();
			break;
		case LOG_TYPE_DXINFO:
			FromFile = new RDxiFile();
			break;
		case LOG_TYPE_DXCLUS:
			FromFile = new RDxClusFile();
			break;
		case LOG_TYPE_ARRL:
			FromFile = new RArrlFile();
			break;
		case LOG_TYPE_ADIF:
			FromFile = new RAdifFile();
			break;
		case LOG_TYPE_UNKNOWN:
		default:
			return FALSE;
	}
        
	FromFile->setFileOptions(&fileOptions);

	if (!FromFile->open(szInputFileName, ios::in))
	{
#if !defined(LC_CMD_LINE)
		char tempMsg[80];
		sprintf(tempMsg, "Error opening %s\n\r", szInputFileName);
		MessageBox(NULL, tempMsg, progName, MB_OK | MB_ICONSTOP);
#else
		printf("Error opening %s\n\r", szInputFileName);
#endif
	}
        
	return TRUE;
}

int RLogConv::getInputLogType()
{
    RFile Log;

#ifdef LC_LINUX
    Log.open(szInputFileName, ios::in);
#else
    Log.open(szInputFileName, ios::in | ios::binary);
#endif
    return Log.fileType();
}

int RLogConv::setOutputFile(char *szOutputFile, int nFileType)
{
	strcpy(szOutputFileName, szOutputFile);

	if (nFileType == 0)
		nFileType = OutputLogType(szOutputFileName);

	switch (nFileType)
	{
		case LOG_TYPE_CT8:
			ToFile = new RCt8File();
			break;
		case LOG_TYPE_TRLOG:
			ToFile = new RTrFile();
			break;
		case LOG_TYPE_DBASE:
			ToFile = new RDbFile();
			break;
		case LOG_TYPE_SDF:
			ToFile = new RSdfFile();
			break;
		case LOG_TYPE_ARRL:
			ToFile = new RArrlFile();
			break;
		case LOG_TYPE_DXINFO:
			ToFile = new RDxiFile();
			break;
		case LOG_TYPE_ADIF:
			ToFile = new RAdifFile();
			break;
		case LOG_TYPE_CABRILLO:
			ToFile = new RCabFile();
			break;
		case LOG_TYPE_CT7:
		case LOG_TYPE_CT9:
		case LOG_TYPE_UNKNOWN:
		default:
			return FALSE;
	}
           
	ToFile->setFileOptions(&fileOptions);

	/* Should have exited from switch statement if object not initialized! */
#ifdef LC_LINUX
	if (!ToFile->open(szOutputFileName, ios::out | ios::noreplace))
#else
	if (!ToFile->open(szOutputFileName, ios::out | ios::noreplace | ios::binary))
#endif
	{
#if !defined(LC_CMD_LINE)                
		MessageBox(NULL, "Output file could not be opened", progName, MB_OK | MB_ICONSTOP);
#else
		printf("Output file could not be opened\n\r");
#endif                
		FromFile->close();
		return FALSE;
	}
            
	if (!(ToFile->writeHeader(FromFile->headerInfo())))
	{
#if !defined(LC_CMD_LINE)            
		MessageBox(NULL, "Contest not supported by output file", progName, MB_OK | MB_ICONSTOP);
#else
		printf("Contest not supported by output file\n\r");
#endif            
		return FALSE;
	}

	return TRUE;
}

int RLogConv::setFileOptions(RQsoFile::QSOFILEOPTIONS *newFileOptions)
{
	if (newFileOptions)
	{
		memcpy(&fileOptions, newFileOptions, sizeof(RQsoFile::QSOFILEOPTIONS));
		return TRUE;
	}
	else
		return FALSE;
}

int RLogConv::setFileOptions(char *cmdLine)
{
    char *tmpPtr;
    
    if (strstr(cmdLine, "-tApproach"))
        fileOptions.time = TIME_APPROACH;

    if (strstr(cmdLine, "-bNumeric"))
        fileOptions.band = BAND_METERS;

    if (strstr(cmdLine, "-freq"))
        fileOptions.fields |= FIELD_FREQ;

    if (strstr(cmdLine, "-qsonum"))
        fileOptions.fields |= FIELD_QSONUM;

    if ((tmpPtr = strstr(cmdLine, "-c")) != NULL)
    {
        tmpPtr += 2;
        for (unsigned char c = 1; c <= MAX_CONTEST; c++)
            if (strncmp(tmpPtr, RQsoFile::contestName[c], strlen(RQsoFile::contestName[c])) == 0)
            {
                fileOptions.contest = c;
                break;
            }
    }

    if ((tmpPtr = strstr(cmdLine, "-n")) != NULL)
    {
        char *cpNote = fileOptions.szNote;
        
        memset(cpNote, '\0', NOTE_SIZE);

        while (*tmpPtr != '"')          // Read up to the next quote character
            tmpPtr++;

        tmpPtr++;
        while (*tmpPtr != '"')          // Keep copying the note field until another
            *cpNote++ = *tmpPtr++;       // quote character is found.
    }

    return TRUE;
}


int RLogConv::convertNextQso(int *position, int *QSOnum)
{
    qso_data qso;

	if (!(FromFile->eof()) && FromFile->nextQso(&qso))
	{
		ToFile->writeQso(&qso);
		*position = FromFile->get_position();
		*QSOnum = ++nQSOsConverted;
		return TRUE;
	}
	else
	{
		return FALSE;
//		if (FromFile->eof())
//		{
//			*position = -1;
//			return TRUE;
//		}
//		else
//			return FALSE;
	}
}

#if !defined(LC_CMD_LINE)
// This routine returns the prefix name of a file supplied in pathName.  For
//  example pathName="d:\tmp\example.txt" would result in name="example".
void RLogConv::FilePrefix(char *name, char *pathName)
{
    char *dot = strrchr(pathName, '.');
    char *slash = strrchr(pathName, '\\');

    if (!dot)
        dot = pathName + strlen(pathName);
    if (!slash)
        slash = pathName;
    else
        slash++;

    strncpy(name, slash, dot - slash);
    name[dot - slash] = NULL;
    return;
}
#endif

// This routine tries to determine the output file's file
// type based on the user supplied file name's suffix.
int RLogConv::OutputLogType(char *sFileName)
{
    char *suffix = strrchr(sFileName, '.');

    if (suffix)
        suffix++;

    if (stricmp(suffix, "bin") == 0)
        return LOG_TYPE_CT8;
    else if (stricmp(suffix, "dbf") == 0)
        return LOG_TYPE_DBASE;
    else if (stricmp(suffix, "dat") == 0)
        return LOG_TYPE_TRLOG;
    else if (stricmp(suffix, "sdf") == 0)
        return LOG_TYPE_SDF;
    else if (stricmp(suffix, "log") == 0)
        return LOG_TYPE_ARRL;
    else if (stricmp(suffix, "adi") == 0)
        return LOG_TYPE_ADIF;
    else if (stricmp(suffix, "cab") == 0)
        return LOG_TYPE_CABRILLO;
    else
        return LOG_TYPE_UNKNOWN;
}

